indexing
   description : "Database Cursor"
   project     : "EiffelBDB - An Eiffel interface to the Berkeley DB"
   author      : "Pirmin Kalberer <kalberer@spin.ch>"
   copyright   : "Copyright (c) 1999, Pirmin Kalberer"
   date        : "$Date: 2000/01/08 16:19:03 $"
   revision    : "$Revision: 1.2 $"
	 
class BDB_C
   
inherit
   BDB_ENV_COMMON
   DB_WRAPPER
   
creation
   make_from_pointer

feature
   
   last_result: INTEGER
	 -- most features set last_result. 0 = successful
   
   last_record: expanded BDB_KEY_VALUE_PAIR
	 -- Last key/value pair retrieved with get
   
feature { BDB_DB }

   make_from_pointer(ptr: POINTER) is
      do
	 dbc_ptr := ptr
	 !!key_.make
	 !!value_.make
      end
   
feature
   
   close is
	 -- C++: int close()
      do
	 last_result := dbc_c_close(dbc_ptr)
	 
	 if last_result /= 0 then
	    runtime_error("BDB_C::close", last_result, Void)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
   
   dup(flags: INTEGER): BDB_C is
	 -- C++: int dup(Dbc** cursorp, u_int32_t flags)
      local
	 ptr: expanded BDB_C_HANDLE[POINTER]
      do
	 last_result := dbc_c_dup(dbc_ptr, ptr.handle, flags)
	 if last_result /= 0 then
	    Result := Void
	    runtime_error("BDB_C::dup", last_result, Void)
	 else
	    !!Result.make_from_pointer(ptr.item)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
	
   del(flags: INTEGER) is
	 -- C++: int del(u_int32_t flags)
      do
	 last_result := dbc_c_del(dbc_ptr, flags)
	 
	 -- Db_keyempty_const is a "normal" return, so should not be
	 -- thrown as an error
	 if last_result /= 0 and then last_result /= Db_keyempty_const then
	    runtime_error("BDB_C::del", last_result, Void)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
	
   first is
	 -- Result available in `last_record'
      do
	 get(Db_first_const)
      ensure
	 last_result_set: -- last_result = 0 implies ok
	 last_record_set: -- last_record.key and last_record.value set
      end
   
   forth is
	 -- Result available in `last_record'
      do
	 get(Db_next_const)
      ensure
	 last_result_set: -- last_result = 0 implies ok
	 last_record_set: -- last_record.key and last_record.value set
      end
   
   go_to(recno: INTEGER) is
	 -- Move cursor to `recno'
 	 -- Result available in `last_record'
      require
	 is_btree_db: -- database must be of type Btree and it must have been created with the DB_RECNUM flag
      do
	 key_.set_recno(recno)
	 get_t(key_, value_, Db_set_recno_const)
	 last_record.set_key(key_.get_data)
	 last_record.set_value(value_.get_data)
      ensure
	 last_result_set: -- last_result = 0 implies ok
	 last_record_set: -- last_record.key and last_record.value set
      end
	
   off: BOOLEAN is
      do
	 Result := (last_result /= 0)
      end
   
   get(flags: INTEGER) is
	 -- Result available in `last_record'
	 -- C++: int get(Dbt* key, Dbt *data, u_int32_t flags)
      do
	 get_t(key_, value_, flags)
	 last_record.set_key(key_.get_data)
	 last_record.set_value(value_.get_data)
      ensure
	 last_result_set: -- last_result = 0 implies ok
	 last_record_set: -- last_record.key and last_record.value set
      end
   
   get_t(key, value: BDB_T; flags: INTEGER) is
	 -- C++: int get(Dbt* key, Dbt *data, u_int32_t flags)
      require
	 key_not_void: key /= Void
	 value_not_void: value /= Void
      do
	 last_result := dbc_c_get(dbc_ptr, key.struct_ptr, value.struct_ptr, flags)
	 
	 -- Db_notfound_const is a "normal" return, so should not be
	 -- thrown as an error
	 if last_result /= 0 and then last_result /= Db_notfound_const then
	    runtime_error("BDB_C::get", last_result, Void)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
   
   put(key, value: STRING; flags: INTEGER) is
	 -- C++: int put(Dbt* key, Dbt *data, u_int32_t flags)
      require
	 key_not_void: key /= Void
	 value_not_void: value /= Void
      do
	 key_.set_data(key)
	 value_.set_data(value)
	 put_t(key_, value_, flags)
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end

   put_t(key, value: BDB_T; flags: INTEGER) is
	 -- C++: int put(Dbt* key, Dbt *data, u_int32_t flags)
      require
	 key_not_void: key /= Void
	 value_not_void: value /= Void
      do
	 last_result := dbc_c_put(dbc_ptr, key.struct_ptr, value.struct_ptr, flags)
	 
	 -- Db_keyexist_const is a "normal" return, so should not be
	 -- thrown as an error
	 if last_result /= 0 and then last_result /= Db_keyexist_const then
	    runtime_error("BDB_C::put", last_result, Void)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end

feature {NONE}
   
   -- Note: use BDB_DB::cursor() to get pointers to a BDB_C,
   -- and call BDB_C::close() rather than delete to release them.
   --
   make is
	 -- C++: Dbc()
      do
      end
	
   exit_Dbc is
	 -- C++: ~Dbc()
      do
      end

feature { NONE } -- Internal

   dbc_ptr: POINTER
	 -- pointer to struct DBC
	 -- memory for struct is allocated by DB::cursor or DBC::c_dup
	
   key_, value_: BDB_T
	 -- Internal variables for DB operations with strings
   
feature { NONE } -- Externals

   dbc_c_close(dbc: POINTER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
   dbc_c_del(dbc: POINTER; flags: INTEGER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
   dbc_c_dup(dbc: POINTER; cursor_p: POINTER; flags: INTEGER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
   dbc_c_get(dbc: POINTER; key, value: POINTER; flags: INTEGER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
   dbc_c_put(dbc: POINTER; key, value: POINTER; flags: INTEGER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
   
   
invariant
   c_struct_initialized: dbc_ptr /= default_pointer
   
end
