indexing
   description : "Database table = a set of keys with associated values"
   project     : "EiffelBDB - An Eiffel interface to the Berkeley DB"
   author      : "Pirmin Kalberer <kalberer@spin.ch>"
   copyright   : "Copyright (c) 1999, Pirmin Kalberer"
   date        : "$Date: 2000/01/08 16:19:03 $"
   revision    : "$Revision: 1.2 $"
	 
class BDB_DB
   
inherit
   BDB_ENV_COMMON
   DB_WRAPPER
   BDB_EXTERNAL_ROUTINES
   
creation
   make
   
feature
   
   last_result: INTEGER
	 -- Most features set last_result. 0 = successful
	 -- Error string is returned by BDB_ENV_COMMON::strerror
   
   last_record: expanded BDB_KEY_VALUE_PAIR
	 -- Last key/value pair retrieved with get or get_recno
   
feature -- creation
   
   make(dbenv: BDB_ENV; flags: INTEGER) is
	 -- create a Db object, then call open
	 -- C++: Db(DbEnv*, u_int32_t)
      do
	 env_ := dbenv
	 last_result := db_create($db_ptr, get_struct_ptr(env_), flags)
	 if last_result /= 0 then
	    runtime_error("BDB_DB::make", last_result, env_)
	 end
	 !!key_.make
	 !!value_.make
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
   
feature
   
   close(flags: INTEGER) is
	 -- C++: int close(u_int32_t flags)
      do
	 last_result := db_close(db_ptr, flags)
	 if last_result /= 0 then
	    runtime_error("BDB_DB::close", last_result, env_)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
   
   cursor(txnid: BDB_TXN; flags: INTEGER): BDB_C is
	 -- C++: int cursor(DbTxn *txnid, Dbc **cursorp, u_int32_t flags)
      local
	 ptr: expanded BDB_C_HANDLE[POINTER]
      do
	 last_result := db_cursor(db_ptr, get_struct_ptr(txnid), ptr.handle, flags)
	 if last_result /= 0 then
	    Result := Void
	    runtime_error("BDB_DB::cursor", last_result, env_)
	 else
	    !!Result.make_from_pointer(ptr.item)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
	
   del(txnid: BDB_TXN; key: STRING; flags: INTEGER) is
	 -- C++: int del(DbTxn *txnid, Dbt *key, u_int32_t flags)
      require
	 key_not_void: key /= Void
      do
	 key_.set_data(key)
	 del_t(txnid, key_, flags)
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
	
   del_t(txnid: BDB_TXN; key: BDB_T; flags: INTEGER) is
	 -- C++: int del(DbTxn *txnid, Dbt *key, u_int32_t flags)
      require
	 key_not_void: key /= Void
      do
	 last_result := db_del(db_ptr, get_struct_ptr(txnid), key.struct_ptr, flags)
	 -- Db_notfound_const is a "normal" return, so should not be
	 -- thrown as an error
	 if last_result /= 0 and then last_result /= Db_notfound_const then
	    runtime_error("BDB_DB::del", last_result, env_)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
	
   fd: INTEGER is
	 -- C++: int fd(int *fdp)
      local   
	 int_result: expanded BDB_C_HANDLE[INTEGER]
      do
	 last_result := db_fd(db_ptr, int_result.handle)
	 if last_result /= 0 then
	    runtime_error("BDB_DB::fd", last_result, env_)
	 else
	    Result := int_result.item
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
	
   get(txnid: BDB_TXN; key, value: STRING; flags: INTEGER) is
	 -- Result available in `last_record'
	 -- C++: int get(DbTxn *txnid, Dbt *key, Dbt *value, u_int32_t flags)
      require
	 key_not_void: key /= Void
	 --value_not_void: value /= Void implies flags=DB_GET_BOTH
      do
	 key_.set_data(key)
	 if value /= Void then
	    value_.set_data(value)
	 end
	 get_t(txnid, key_, value_, flags)
	 last_record.set_key(key_.get_data)
	 last_record.set_value(value_.get_data)
      ensure
	 last_result_set: -- last_result = 0 implies ok
	 last_record_set: -- last_record.key and last_record.value set
      end
	
   get_value(txnid: BDB_TXN; key: STRING; flags: INTEGER): STRING is
	 -- C++: int get(DbTxn *txnid, Dbt *key, Dbt *value, u_int32_t flags)
      require
	 key_not_void: key /= Void
      do
	 key_.set_data(key)
	 get_t(txnid, key_, value_, flags)
	 Result := value_.get_data
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
	
   get_recno(txnid: BDB_TXN; recno: INTEGER; flags: INTEGER) is
	 -- Result available in `last_record'
      do
	 key_.set_recno(recno)
	 get_t(txnid, key_, value_, flags)
	 if last_result = 0 then
	     last_record.set_key(key_.get_data)
	     last_record.set_value(value_.get_data)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
	 last_record_set: -- last_record.key and last_record.value set
      end
	
   get_t(txnid: BDB_TXN; key, value: BDB_T; flags: INTEGER) is
	 -- C++: int get(DbTxn *txnid, Dbt *key, Dbt *value, u_int32_t flags)
      require
	 key_not_void: key /= Void
	 value_not_void: value /= Void
      do
	 last_result := db_get(db_ptr, get_struct_ptr(txnid), key.struct_ptr, value.struct_ptr, flags)
	 -- Db_notfound_const is a "normal" return, so should not be
	 -- thrown as an error
	 if last_result /= 0 and then last_result /= Db_notfound_const then
	    runtime_error("BDB_DB::get", last_result, env_)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
	
   join(curslist: ARRAY[BDB_C]; flags: INTEGER): BDB_C is
	 -- C++: int join(Dbc **curslist, u_int32_t flags, Dbc **dbcp)
      do
	 --TODO
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
	
   put(txnid: BDB_TXN; key, value: STRING; flags: INTEGER) is
	 -- C++: int put(DbTxn *txnid, Dbt *key, Dbt *value, u_int32_t flags)
      require
	 key_not_void: key /= Void
	 value_not_void: value /= Void
      do
	 key_.set_data(key)
	 value_.set_data(value)
	 put_t(txnid, key_, value_, flags)
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
	
   put_recno(txnid: BDB_TXN; recno: INTEGER; value: STRING) is
      require
	 value_not_void: value /= Void
      do
	 key_.set_recno(recno)
	 value_.set_data(value)
	 put_t(txnid, key_, value_, 0)
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
	
   put_last(txnid: BDB_TXN; value: STRING): INTEGER is
      require
	 value_not_void: value /= Void
      do
	 key_.set_recno(0)
	 value_.set_data(value)
	 put_t(txnid, key_, value_, Db_append_const)
	 Result := key_.get_recno
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
	
   put_t(txnid: BDB_TXN; key, value: BDB_T; flags: INTEGER) is
	 -- C++: int put(DbTxn *txnid, Dbt *key, Dbt *value, u_int32_t flags)
      require
	 key_not_void: key /= Void
	 value_not_void: value /= Void
      do
	 last_result := db_put(db_ptr, get_struct_ptr(txnid), key.struct_ptr, value.struct_ptr, flags)
	 -- Db_keyexist_const is a "normal" return, so should not be
	 -- thrown as an error
	 if last_result /= 0 and then last_result /= Db_keyexist_const then
	    runtime_error("BDB_DB::put", last_result, env_)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
	
   stat(flags: INTEGER): STRING is
	 -- C++: int stat(void *sp, void *(*db_malloc)(size_t), u_int32_t flags)
      local
	 ptr: expanded BDB_C_HANDLE[POINTER]
      do
	 last_result := db_stat(db_ptr, ptr.handle, default_pointer, flags)
	 if last_result /= 0 then
	    runtime_error("BDB_DB::stat", last_result, env_)
	 else
	    --TODO: make string from ptr.item structure
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
	
   sync(flags: INTEGER) is
	 -- C++: int sync(u_int32_t flags)
      do
	 last_result := db_sync(db_ptr, flags)
	 if last_result /= 0 then
	    runtime_error("BDB_DB::sync", last_result, env_)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end

   get_byteswapped: INTEGER is
	 -- C++: int get_byteswapped() const
      do
	 --TODO: Result := db_get_byteswapped(db_ptr)
      end
	
   get_type: INTEGER is
	 -- C++: DBTYPE get_type() const
      do
	 --TODO: Result := db_get_type(db_ptr)
      end

   open(name, subname: STRING; type: INTEGER; flags: INTEGER;
	mode: INTEGER) is
	 -- C++: int open(const char *name, const char *subname,
	 --          DBTYPE type, u_int32_t flags, int mode)
      do
	 last_result := db_open(db_ptr, string_to_pointer(name), string_to_pointer(subname), type, flags, mode)
	 if last_result /= 0 then
	    runtime_error("BDB_DB::open", last_result, env_)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end

   set_flags(flags: INTEGER) is
      do
	 last_result := db_set_flags(db_ptr, flags)
	 if last_result /= 0 then
	    runtime_error("BDB_DB::set_flags", last_result, env_)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
   
   set_pagesize(pagesize: INTEGER) is
      do
	 last_result := db_set_pagesize(db_ptr, pagesize)
	 if last_result /= 0 then
	    runtime_error("BDB_DB::set_pagesize", last_result, env_)
	 end
      ensure
	 last_result_set: -- last_result = 0 implies ok
      end
   
feature { NONE } -- Internal
   
   db_ptr: POINTER
	 -- pointer to struct DB
	 -- memory for struct is allocated by DB::create
   
   env_: BDB_ENV

   key_, value_: BDB_T
	 -- Internal variables for DB operations with strings
   
feature { NONE } -- Externals

   db_open(db: POINTER; name, subname: POINTER; type, flags, mode: INTEGER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
   db_close(db: POINTER; flags: INTEGER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
   db_cursor(db: POINTER; txn: POINTER; cursor_p: POINTER; flags: INTEGER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
   db_del(db: POINTER; txn: POINTER; key: POINTER; flags: INTEGER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
   db_fd(db: POINTER; fdp: POINTER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
   db_get(db: POINTER; txn: POINTER; key, value: POINTER; flags: INTEGER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
   db_join(db: POINTER; c1_p: POINTER; flags: INTEGER; c2_p: POINTER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
   db_put(db: POINTER; txn: POINTER; key, value: POINTER; flags: INTEGER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
   db_stat(db: POINTER; sp: POINTER; malloc_f: POINTER; flags: INTEGER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
   db_sync(db: POINTER; flags: INTEGER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
   
   db_set_flags(db: POINTER; flags: INTEGER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
   db_set_pagesize(db: POINTER; size: INTEGER): INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
invariant
   c_struct_initialized: db_ptr /= default_pointer
      
end

