indexing
   description : ""
   project     : "EiffelBDB - An Eiffel interface to the Berkeley DB"
   author      : "Pirmin Kalberer <kalberer@spin.ch>"
   copyright   : "Copyright (c) 1999, Pirmin Kalberer"
   date        : "$Date: 2000/01/08 16:19:03 $"
   revision    : "$Revision: 1.2 $"
	 
--
-- Base application class.  Provides functions for opening a database.
-- User of this library can use this class as a starting point for
-- developing a DB application - derive their application class from
-- this one, add application control logic.
--

class BDB_ENV
   
inherit
   BDB_ENV_COMMON
   DB_WRAPPER
   BDB_WRAPPED_STRUCT -- struct ENV
      rename
	 make as make_struct
      end

creation
   make
    
feature

    dispose is
    -- C++: ~DbEnv()
	do
    -- We want to call appexit() to enforce proper cleanup when
    -- a BDB_ENV goes out of scope or is deleted.  At the same
    -- time, we want to permit the user to call appexit() at
    -- any time, so they can check the error return and/or
    -- shut down DB at any time.  DB normally does not allow
    -- multiple appexit() calls, but to satisfy the above needs,
    -- we'll look at the (normally internal) flag DB_ENV_APPINIT,
    -- that is set on appinit(), and cleared on appexit().
	   --TODO
 	end

    -- This constructor can be used to immediately initialize the
    -- application with these arguments.  Do not use it if you
    -- need to set other parameters via the access methods.
    --
    make_appinit(homeDir: STRING; db_config: STRING; flags: INTEGER) is
	 -- C++: DbEnv(const char *homeDir, char *const *db_config, u_int32_t flags)
      local
	 err: INTEGER
      do
	 make
--	 err := appinit(homeDir, db_config, flags)
	 if err /= 0 then
	    runtime_error("BDB_ENV::DbEnv", err, Current)
	 end
	end

    -- Use this constructor if you wish to *delay* the initialization
    -- of the db library.  This is useful if you need to set
    -- any particular parameters via the access methods below.
    -- Then call appinit() to complete the initialization.
    --
	make is
    -- C++: DbEnv()
      do
	 set_error_model(Error_model_Exception)
         env_ptr := alloc_db_env
 	end

    ----------------------------------------------------------------
    -- simple get/set access methods
    --
    -- If you are calling set_ methods, you need to
    -- use the default constructor along with appinit().

    set_lorder(value: INTEGER) is
    -- Byte order.
    -- C++: void set_lorder(int)
      do
	 -- TODO
	   --if (flags & DB_ENV_APPINIT) != 0 then
	   --   runtime_error("BDB_ENV::set_", EINVAL, 0, 1);
	   --end
	   --env._field := value
	end

    set_paniccall(fcn: POINTER) is
    -- Panic callback.
    -- C++: void set_paniccall(db_paniccall_fcn)
	do
	   --db_env_db_panic_set(env, fcn)
	end

    set_errcall(fcn: POINTER) is
    -- Error message callback.
    -- C++: void set_errcall(db_errcall_fcn)
	do
	   -- TODO (like set_lorder)
	end

    set_errfile(p1 : POINTER) is
    -- Error message file stream.
    -- C++: void set_errfile(FILE *)
	do
	   -- TODO (like set_lorder)
	end

    set_errpfx(errpfx: STRING) is
    -- Error message prefix.
    -- C++: void set_errpfx(const char *)
	do
	   -- TODO (WO_ACCESS)
	end

    set_verbose(verbose: INTEGER) is
    -- Generate debugging messages.
    -- C++: void set_verbose(int)
	do
	   -- TODO (like set_lorder)
	end

    ----------------------------------------------------------------
    -- Locking.

    get_lk_info: POINTER is
    -- Return from lock_open().
    -- C++: DbLockTab *get_lk_info() const
	do
	   -- TODO
	end

    set_lk_conflicts(lk_conflicts: INTEGER) is
    -- Two dimensional conflict matrix.
    -- C++: void set_lk_conflicts(u_int8_t *)
	do
	   -- TODO (like set_lorder)
	end

    set_lk_modes(lk_modes: INTEGER) is
    -- Number of lock modes in table.
    -- C++: void set_lk_modes(int)
	do
	   -- TODO (like set_lorder)
	end

    set_lk_max(lk_max: INTEGER) is
    -- Maximum number of locks.
    -- C++: void set_lk_max(u_int32_t)
	do
	   -- TODO (like set_lorder)
	end

    set_lk_detect(lk_detect: INTEGER) is
    -- Deadlock detect on every conflict.
    -- C++: void set_lk_detect(u_int32_t)
	do
	   -- TODO (like set_lorder)
	end


    ----------------------------------------------------------------
    -- Logging.

    get_lg_info: POINTER is
    -- Return from log_open().
    -- C++: DbLog *get_lg_info() const
	do
	   -- TODO
	end

    set_lg_max(lg_max: INTEGER) is
    -- Maximum file size.
    -- C++: void set_lg_max(u_int32_t)
	do
	   -- TODO (like set_lorder)
	end


    ----------------------------------------------------------------
    -- Memory pool.

    get_mp_info: POINTER is
    -- Return from memp_open().
    -- C++: DbMpool *get_mp_info() const
	do
	   -- TODO
	end

    set_mp_mmapsize(mp_mmapsize: INTEGER) is
    -- Maximum file size for mmap.
    -- C++: void set_mp_mmapsize(size_t)
	do
	   -- TODO (like set_lorder)
	end

    set_mp_size(mp_size: INTEGER) is
    -- Bytes in the mpool cache.
    -- C++: void set_mp_size(size_t)
	do
	   -- TODO (like set_lorder)
	end


    ----------------------------------------------------------------
    -- Transactions.

    get_tx_info: POINTER is
    -- Return from txn_open().
    -- C++: DbTxnMgr *get_tx_info() const
	do
	   -- TODO
	end

    set_tx_max(tx_max: INTEGER) is
    -- Maximum number of transactions.
    -- C++: void set_tx_max(u_int32_t)
	do
	   -- TODO (like set_lorder)
	end

    set_tx_recover(fcn: POINTER) is
    -- Dispatch function for recovery.
    -- C++: void set_tx_recover(tx_recover_fcn)
	do
	   -- TODO (like set_lorder)
	end

    set_flags(flags: INTEGER) is
    -- Flags.
    -- C++: void set_flags(u_int32_t)
	do
	   -- TODO (like set_lorder)
	end

    ----------------------------------------------------------------
    -- The default error model is to throw an exception whenever
    -- an error occurs.  This generally allows for cleaner logic
    -- for transaction processing, as a try block can surround a
    -- single transaction.  Alternatively, since almost every method
    -- returns an error code (errno), the error model can be set to
    -- not throw exceptions, and instead return the appropriate code.
    --
	Error_model_Exception, Error_model_ErrorReturn: INTEGER is unique
    -- C++: enum ErrorModel { Exception, ErrorReturn }
	
    set_error_model(model : INTEGER) is
    -- C++: void set_error_model(ErrorModel)
	do
	   error_model_ := model	
	end

    -- If an error is detected and the error call function
    -- or stream is set, a message is dispatched or printed.
    -- If a prefix is set, each message is prefixed.
    --
    -- You can use set_errcall() or set_errfile() above to control
    -- error functionality using a C model.  Alternatively, you can
    -- call set_error_stream() to force all errors to a C++ stream.
    -- It is unwise to mix these approaches.
    --
	set_error_stream(stream: POINTER) is
    -- C++: void set_error_stream(class ostream*)
      do
	 -- TODO
	end

    -- used internally
	runtime_exception(caller: STRING; err: INTEGER) is
      do
	 --TODO
	end

feature {NONE}
    -- We can add data to this class if needed
    -- since parent class is not allocated by db.
    -- (see comment at top)

    error_model_: INTEGER
    -- C++: ErrorModel error_model_

    stream_error_function(prefix_str, message: STRING) is
    -- C++: static void stream_error_function(const char *, char *)
      do
	 --TODO
	end
	
	--TODO: ostream *error_stream_;
    -- C++: static ostream *error_stream_
     
feature { NONE } --etc.

   env_ptr: POINTER
     
feature { NONE } -- Externals

end

