indexing
   description : "A chunk of data, maybe a key or value"
   project     : "EiffelBDB - An Eiffel interface to the Berkeley DB"
   author      : "Pirmin Kalberer <kalberer@spin.ch>"
   copyright   : "Copyright (c) 1999, Pirmin Kalberer"
   date        : "$Date: 2000/01/08 16:19:03 $"
   revision    : "$Revision: 1.2 $"

class BDB_T
   
inherit
   DB_WRAPPER
   BDB_EXTERNAL_ROUTINES
   BDB_WRAPPED_STRUCT -- struct DBT
      rename
	 make as make_struct
      end
   
creation
   make,
   make_from_string
   
feature

   make is
	 -- C++: Dbt()
      do
	 make_struct(c_sizeof_dbt)
      end
      
   make_from_string(str: STRING) is
	 -- C++: Dbt(void *data, size_t size)
      do
	 make_struct(c_sizeof_dbt)
	 set_data(str)
	 set_size(str.count)
      end
	
feature

   -- key/data
   get_data: STRING is
	 -- new string is allocated whith each call
	 -- C++: void *get_data() const
      local
	 data_str: BDB_STRING
      do
	 !!data_str.make_from_dbt(db_dbt_data_get(struct_ptr), db_dbt_size_get(struct_ptr))
	 -- The BDB_STRING is returned as "pure" STRING
	 -- because otherwise you get a runtime error when
	 -- doing something like get_data.is_equal("1234")
	 Result := data_str.out
      end
   
   get_recno: INTEGER is
      do
	 Result := recno
      end
   
   set_data(data: STRING) is
	 -- C++: void set_data(void *)
      require
	 data_not_void: data /= Void
      local
	 val: POINTER
      do
	 val := db_dbt_data_set(struct_ptr, string_to_pointer(data))
	 set_size(data.count)
      end
   
   set_recno(no: INTEGER) is
      local
	 val: POINTER
      do
	 recno := no
	 val := db_dbt_data_set(struct_ptr, $recno)
	 set_size(4) --FIXME: sizeof(db_recno_t)
      end
	 
   get_size: INTEGER is
	 -- key/data length
	 -- C++: u_int32_t get_size() const
      do
	 Result := db_dbt_size_get(struct_ptr)
      end
   
   set_size(size: INTEGER) is
	 -- C++: void set_size(u_int32_t)
      local
	 val: INTEGER
      do
	 val := db_dbt_size_set(struct_ptr, size)
      end

   get_ulen: INTEGER is
	 -- RO: length of user buffer.
	 -- C++: u_int32_t get_ulen() const
      do
	 Result := db_dbt_ulen_get(struct_ptr)
      end
	
   set_ulen(ulen: INTEGER) is
	 -- C++: void set_ulen(u_int32_t)
      local
	 val: INTEGER
      do
	 val := db_dbt_ulen_set(struct_ptr, ulen)
      end

   get_dlen: INTEGER is
	 -- RO: get/put record length.
	 -- C++: u_int32_t get_dlen() const
      do
	 Result := db_dbt_dlen_get(struct_ptr)
      end
   
   set_dlen(dlen: INTEGER) is
	 -- C++: void set_dlen(u_int32_t)
      local
	 val: INTEGER
      do
	 val := db_dbt_dlen_set(struct_ptr, dlen)
      end

   get_doff: INTEGER is
	 -- RO: get/put record offset.
	 -- C++: u_int32_t get_doff() const
      do
	 Result := db_dbt_doff_get(struct_ptr)
      end
	
   set_doff(doff: INTEGER) is
	 -- C++: void set_doff(u_int32_t)
      local
	 val: INTEGER
      do
	 val := db_dbt_doff_set(struct_ptr, doff)
      end

   get_flags: INTEGER is
	 -- flags
	 -- C++: u_int32_t get_flags() const
      do
	 Result := db_dbt_flags_get(struct_ptr)
      end
	
   set_flags(flags: INTEGER) is
	 -- C++: void set_flags(u_int32_t)
      local
	 val: INTEGER
      do
	 val := db_dbt_flags_set(struct_ptr, flags)
      end

   exit_Dbt is
	 -- C++: ~Dbt()
      do
      end
	
   make_from_other(other: like Current) is
	 -- C++: Dbt(const Dbt &)
      do
	 --TODO
      end
	
   copy_(other: like Current) is
	 -- C++: Dbt &operator = (const Dbt &)
      do
	 --TODO
      end
   
feature { NONE } -- Internal
   
   recno: INTEGER --FIXME: type db_recno_t
	 -- used for recno key
   

feature { NONE } -- Externals

   c_sizeof_dbt: INTEGER is
      external "C_InlineWithoutCurrent"
      end
      
end

