/*
   description : "Additional C functions"
   project     : "EiffelBDB - An Eiffel interface to the Berkeley DB"
   author      : "Pirmin Kalberer <kalberer@spin.ch>"
   copyright   : "Copyright (c) 1999, Pirmin Kalberer"
   date        : "$Date: 2000/01/08 16:19:03 $"
   revision    : "$Revision: 1.2 $"
*/

#include <stdlib.h>
#include "db.h"

int c_sizeof_dbt(void)
{
  return sizeof(DBT);
}

/* DB access method description structure. */

/* ------------ struct __db ------------ */

int db_open(void *db, const char * name, const char * subname, DBTYPE type, u_int32_t flags, int mode)
{
  return ((DB *)db)->open((DB *)db, (const char *)name, (const char *)subname, (DBTYPE)type, (u_int32_t)flags, mode);
}

int db_close(void *db, int flags)
{
  return ((DB *)db)->close((DB *)db, (u_int32_t)flags);
}

int db_cursor(void *db, void *txn, void *cursor_p, int flags)
{
  return ((DB *)db)->cursor((DB *)db, (DB_TXN *)txn, (DBC **)cursor_p, (u_int32_t)flags);
}

int db_del(void *db, void *txn, void *key, int flags)
{
  return ((DB *)db)->del((DB *)db, (DB_TXN *)txn, (DBT *)key, (u_int32_t)flags);
}

int db_fd(void *db, void *fd)
{
  return ((DB *)db)->fd((DB *)db, (int *)fd);
}

int db_get(void *db, void *txn, void *key, void *data, int flags)
{
  return ((DB *)db)->get((DB *)db, (DB_TXN *)txn, (DBT *)key, (DBT *)data, (u_int32_t)flags);
}

int db_join(void *db, void *c1_p, int flags, void *c2_p)
{
  return ((DB *)db)->join((DB *)db, (DBC **)c1_p, (u_int32_t)flags, (DBC **)c2_p);
}

int db_put(void *db, void *txn, void *key, void *data, int flags)
{
  return ((DB *)db)->put((DB *)db, (DB_TXN *)txn, (DBT *)key, (DBT *)data, (u_int32_t)flags);
}

int db_stat(void *db, void *a1, void *f, int flags)
{
  return ((DB *)db)->stat((DB *)db, a1, NULL, (u_int32_t)flags); /* FIXME */
}

int db_sync(void *db, int flags)
{
  return ((DB *)db)->sync((DB *)db, (u_int32_t)flags);
}

int db_set_flags(void *db, int flags)
{
  return ((DB *)db)->set_flags((DB *)db, (u_int32_t)flags);
}

int db_set_pagesize(void *db, int size)
{
  return ((DB *)db)->set_pagesize((DB *)db, (u_int32_t)size);
}

/* ------------ struct __dbc ------------ */

int dbc_c_close(void *dbc)
{
  return ((DBC *)dbc)->c_close((DBC *)dbc);
}

int dbc_c_del(void *dbc, int flags)
{
  return ((DBC *)dbc)->c_del((DBC *)dbc, (u_int32_t)flags);
}

int dbc_c_dup(void *dbc, void *cursor_p, int flags)
{
  return ((DBC *)dbc)->c_dup((DBC *)dbc, (DBC **)cursor_p, (u_int32_t)flags);
}

int dbc_c_get(void *dbc, void *key, void *data, int flags)
{
  return ((DBC *)dbc)->c_get((DBC *)dbc, (DBT *)key, (DBT *)data, (u_int32_t)flags);
}

int dbc_c_put(void *dbc, void *key, void *data, int flags)
{
  return ((DBC *)dbc)->c_put((DBC *)dbc, (DBT *)key, (DBT *)data, (u_int32_t)flags);
}
