indexing
   description : "Example for using just the DB access methods"
   project     : "EiffelBDB - An Eiffel interface to the Berkeley DB"
   author      : "Pirmin Kalberer <kalberer@spin.ch>"
   copyright   : "Copyright (c) 1999, Pirmin Kalberer"
   date        : "$Date: 2000/01/08 16:19:08 $"
   revision    : "$Revision: 1.2 $"

class ACCESS_EXAMPLE
   
creation
   make
   
feature { NONE }
   
   filename: STRING is "access.db"
   subname: STRING -- default = Void -> no subname
   
   wordlist: ARRAY[STRING] is
      once
	 Result := <<"Wall", "Larry",
		     "Smith", "John",
		     "mouse", "mickey",
		     "duck", "donald",
		     >>
      end
		     
feature
   
   make is
      local
	 db: BDB_DB
	 env: BDB_ENV -- default = Void -> no environment
	 txn: BDB_TXN -- default = Void -> no transaction handling
	 key, data: STRING
	 key_t, data_t: BDB_T
	 dbcp: BDB_C
	 no_flags: INTEGER
	 i: INTEGER
      do
	 -- Create the database object.
	 -- There is no environment for this simple example.
	 !!db.make(env, no_flags)
      
	 --db.set_error_stream(&cerr)
	 --db.set_errpfx("AccessExample")
	 db.set_pagesize(1024)          -- Page size: 1K.
	 --db.set_cachesize(0, 32 * 1024, 0)
		   
	 db.open(filename, subname, db.Db_btree_const, db.Db_create_const + db.Db_truncate_const, 420) -- 420 = octal 644
	 
	 -- Insert records into the database.
	 from
	    i := wordlist.lower
	 until
	    i > wordlist.upper
	 loop
	    db.put(txn, wordlist.item(i), wordlist.item(i+1), db.Db_nooverwrite_const)
	    if db.last_result = db.Db_keyexist_const then
	       print("Key "); print(wordlist.item(i)); print(" already exists.%N")
	    end
	    i := i + 2
	 end
	 
	 -- Acquire a cursor for the table.
	 dbcp := db.cursor(txn, no_flags)
         
	 -- Walk through the table, printing the key/data pairs.
	 from
	    dbcp.first
	 until 
	    dbcp.off
	 loop
	    print(dbcp.last_record.key); print(": "); print(dbcp.last_record.value); print("%N")
	    dbcp.forth
	 end
	 dbcp.close
	 
	 -- search and print one key/data pair.	 
	 !!key.make_from_string("duck")
	 data := db.get_value(txn, key, no_flags)
	 if db.last_result = db.Db_notfound_const then
	    print(key); print(" not found%N")
	 else
	    print(key); print(": "); print(data); print("%N")
	 end
	 
	 -- search and print one key/data pair.	Use of BDB_T class.
	 !!key_t.make_from_string("duck")
	 !!data_t.make
	 db.get_t(txn, key_t, data_t, no_flags)
	 if db.last_result = db.Db_notfound_const then
	    print(key_t.get_data); print(" not found%N")
	 else
	    print(key_t.get_data); print(": "); print(data_t.get_data); print("%N")
	 end
	 
	 db.close(no_flags)
      end -- make
   
end
