indexing
   description : "Example for using the BTREE access method with record numbers"
   project     : "EiffelBDB - An Eiffel interface to the Berkeley DB"
   author      : "Pirmin Kalberer <kalberer@spin.ch>"
   copyright   : "Copyright (c) 1999, Pirmin Kalberer"
   date        : "$Date: 2000/01/08 16:19:08 $"
   revision    : "$Revision: 1.1 $"

class BT_REC_EXAMPLE
   
creation
   make
   
feature { NONE }
   
   filename: STRING is "access.db"
   subname: STRING -- default = Void -> no subname
   
   wordlist: ARRAY[STRING] is
      once
	 Result := <<"Wall", "Larry",
		     "Smith", "John",
		     "mouse", "mickey",
		     "duck", "donald",
		     >>
      end
		     
feature
   
   make is
      local
	 db: BDB_DB
	 env: BDB_ENV -- default = Void -> no environment
	 txn: BDB_TXN -- default = Void -> no transaction handling
	 key, data: STRING
	 dbcp: BDB_C
	 no_flags: INTEGER
	 cnt, i: INTEGER
      do
	 !!db.make(env, no_flags)
      
	 --db.set_error_stream(&cerr)
	 --db.set_errpfx("BtRecExample")
	 db.set_pagesize(1024)          -- 1K page sizes.
	 
	 db.set_flags(db.Db_recnum_const)  -- Record numbers.
		   
	 db.open(filename, subname, db.Db_btree_const, db.Db_create_const + db.Db_truncate_const, 420) -- 420 = octal 644
	 
	 -- Insert records into the database, where the key is the word
	 -- preceded by its record number, and the data is the same, but
	 -- in reverse order.
	 from
	    cnt := 1
	 until
	    cnt > 1000
	 loop
	    key := cnt.out
	    key.append(wordlist.item(((cnt-1) \\ wordlist.count) + 1))
	    
	    !!data.make(key.count)
	    from i := key.count until i = 1 loop
	       data.append_character(key.item(i))
	       i := i - 1
	    end
		       
	    db.put(txn, key, data, db.Db_nooverwrite_const)
	    if db.last_result = db.Db_keyexist_const then
	       print("Key "); print(key); print(" already exists.%N")
	    end
	    cnt := cnt + 1
	 end
	 
	 -- Acquire a cursor for the table.
	 dbcp := db.cursor(txn, no_flags)
         
	 -- Retrieve and print records 990..1000
	 from
	    dbcp.go_to(990)
	 until 
	    dbcp.off
	 loop
	    print(dbcp.last_record.key); print(": "); print(dbcp.last_record.value); print("%N")
	    dbcp.forth
	 end
	 dbcp.close
	 
	 -- Search and print one record
	 db.get_recno(txn, 999, db.Db_set_recno_const)
	 if db.last_result = db.Db_notfound_const then
	    print("Record not found%N")
	 else
	    print(db.last_record.key); print(": "); print(db.last_record.value); print("%N")
	 end
	 
	 db.close(no_flags)
      end -- make
   
end
