indexing

   description : "Portable features for handling external types"
   project     : "EiffelBDB - An Eiffel interface to the Berkeley DB"
   author      : "Pirmin Kalberer <kalberer@spin.ch>"
   copyright   : "Copyright (c) 1999, Pirmin Kalberer"
   date        : "$Date: 1999/11/28 18:58:20 $"
   revision    : "$Revision: 1.1 $"

class BDB_EXTERNAL_ROUTINES

feature -- conversion of Eiffel to C types
   
   any_to_pointer(arg: ANY): POINTER is
      do
	 if arg /= Void then
	    Result := arg.to_pointer
	 end
      end
   
   string_to_pointer(arg: STRING): POINTER is
	 -- A reference to a C form of string `arg'
	 -- Warning: Memory is shared between Eiffel and C
      do
	 if arg /= Void then
	    Result := arg.to_external
	 end
      end
   
   string_to_pointer_copy(arg: STRING): POINTER is
	 -- Create a new C string copied from `arg'
	 -- Warning: Memory of C string is not collected by Eiffel GC
      do
	 if arg /= Void then
	    Result := c_strdup(arg.to_external)
	 end
      end
   
feature -- conversion of C to Eiffel types
   
   pointer_to_string(arg: POINTER): STRING is
	 -- Warning: Memory is shared between Eiffel and C
      do
         if arg /= default_pointer then
	    !!Result.from_external(arg)
	 end
      end
   
   pointer_to_string_copy(arg: POINTER): STRING is
      do
         if arg /= default_pointer then
	    !!Result.from_external_copy(arg)
	 end
      end
   
feature
   
   get_pointer(arg: POINTER): POINTER is
	 -- Return `arg' as POINTER type
	 -- Example: ptr := get_pointer($print)
      do
	 Result := arg
      end
   
feature {NONE} -- external C features
   
   c_strdup(s: POINTER): POINTER is
      external "C_InlineWithoutCurrent"
      alias "strdup"
      end
	 
end
